/*
*  PgpSdk1.h
*  Copyright 1999-2003 by Benchmark Software
*  All Rights Reserved
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilver.skuz.net 
*
*  Richard
*  --
*  R.Christman
*  Benchmark Software
*  quicksilver@skuz.net
*  http://quicksilver.skuz.net
*
*/
//---------------------------------------------------------------------------
#ifndef PgpSdk1H
#define PgpSdk1H

#include <pgpKeys.h>
#include <pgpErrors.h>
#include <pgpUtilities.h>
#include <pgpSDKPrefs.h>
#include <pgpFeatures.h>
#include <pgpOptionList.h>
#include <pgpMemoryMgr.h>
#include "PGPbuffers.h"
#include "PgpMsg.h"
#include "global.h"
//---------------------------------------------------------------------------
bool __fastcall HasSdk();
int  __fastcall SdkVer();
int  __fastcall SdkApiVer();
/*
* SDK API version
* example: 1.0.0 = 0x01000000
*   0x01000000   (16777216)   SDK 1.0.0   PGP550
*   0x02000000   (33554432)   SDK 1.1.0
*   0x02000010   (33554448)   SDK 1.1.1   PGP553
*   0x03000000   (50331648)   SDK 1.5.0   PGP600
*   0x03000010   (50331664)   SDK 1.5.1
*   0x03000020   (50331680)   SDK 1.5.2   PGP602
*   0x03001000   (50335744)   SDK 1.6.0
*   0x03002000   (50339840)   SDK 1.7.0   PGP650
*   0x03002010   (50339856)   SDK 1.7.1   PGP651,2,3
*   0x--------   (50339856)   SDK 1.7.8   PGP658
*/

#define PGP600 50331648
#define SDK150 50331648

class TPGPsdk
{
public:
   TForm *owner;
   int decrypts;
   PGPError err;
   HINSTANCE hSdkLib;
   PGPContextRef context;
   PGPUInt32 apiversion,sdkversion;
   TStringList *siginfos,*headers; // utility *headers
   TPGPkeylist *pubkeys;
   char *pubringpkr,*secringskr;
   __fastcall TPGPsdk(TForm *Owner, HINSTANCE &hPgpSdk);
   __fastcall~TPGPsdk();
   bool __fastcall ExportDefaultKeys(TStrings *exports);
   bool __fastcall ExportDefaultKeysFast(TStrings *exports);
   PGPError __fastcall ExportKeySet(PGPKeySetRef keys, PGPExportFormat &ExportFormat,
      bool IncludePrivates, TStrings *exports);
   PGPError __fastcall GetDisableKeyIds(PGPKeySetRef keyset, TStrings *keyids);
   PGPError __fastcall GetRevokedKeyIds(PGPKeySetRef keyset, TStrings *keyids);
   bool __fastcall CheckPGP6NeedsUpdate(TControl *ForHourGlass);
   bool __fastcall UpdateQSKeyrings(TControl *ForHourGlass);
   bool __fastcall UpdateQSKeyringsFast(TControl *ForHourGlass);
   bool __fastcall UpdateQSKeyringsSlow(TControl *ForHourGlass);
   bool __fastcall VerifyClearSigned(TStrings *SignedText);
   PGPError __fastcall DecryptMessage(TStrings *Message, char *Passphrase, TStrings *DecKeyIds, bool recurse);
   PGPError __fastcall DecryptBuffer(TMemoryStream *Buffer, char *Passphrase, bool recurse);
   // new stuff
   PGPError __fastcall SignMessage(TStrings *Message, char *keyid, char *Passphrase);
   PGPError __fastcall SignBuffer(TMemoryStream *Buffer, char *keyid, char *Passphrase);

   PGPError __fastcall EncryptMessage(TStrings *Message, TStrings *KeyIds);
   PGPError __fastcall EncryptBuffer(TMemoryStream *Buffer, TStrings *KeyIds);

   PGPError __fastcall CEncryptMessage(TStrings *Message, char *Passphrase);
   PGPError __fastcall CEncryptBuffer(TMemoryStream *Buffer, char *Passphrase);

   PGPError __fastcall EncryptSignMessage(TStrings *Message, TStrings *EncKeyIds,
      char *SignKeyid, char *Passphrase);
   PGPError __fastcall EncryptSignBuffer(TMemoryStream *Buffer, TStrings *EncKeyIds,
      char *SignKeyid, char *Passphrase);
   // utility
   bool __fastcall IsPgpError(PGPError err, char *errmsg);
   AnsiString __fastcall MessageDecIdText8(TStrings *message);
   void __fastcall GetMessageStrLF(AnsiString &MsgStr, TStrings *Message);
   void __fastcall PushHeaders(TStrings *message); // add headers back to message
   void __fastcall GrabHeaders(TStrings *message); // remove headers from message
   void __fastcall CreateTempPubringFile(); // created temp ring with pubs and rems
   void __fastcall DeleteTempPubringFile();

private:
   PGPError __fastcall PGPsdkInit();
   PGPError __fastcall PGPsdkCleanup();
   PGPError __fastcall PGPNewContext(PGPUInt32 clientAPIVersion, PGPContextRef *newContext);
   PGPError __fastcall PGPsdkLoadDefaultPrefs(PGPContextRef context);
   PGPError __fastcall PGPsdkPrefGetFileSpec(PGPContextRef context, PGPsdkPrefSelector selector, PGPFileSpecRef * outRef);
   PGPError __fastcall PGPGetFullPathFromFileSpec(PGPFileSpecRef fileRef, char **fullPathPtr);
   PGPError __fastcall PGPGetErrorString(PGPError theError, PGPSize bufferSize, char *theString);
   PGPError __fastcall PGPGetSDKVersion(PGPUInt32 *version);
   PGPError __fastcall PGPFreeFileSpec(PGPFileSpecRef fileRef);
   PGPError __fastcall PGPExportKeySet(PGPKeySetRef keys, PGPOptionListRef firstOpt,
      PGPOptionListRef secondOpt,PGPOptionListRef thirdOpt, PGPOptionListRef forthOpt,
      PGPOptionListRef fifthOpt);
   PGPError __fastcall PGPFreeKeySet(PGPKeySetRef keys);
   PGPError __fastcall PGPFreeData(void *allocation);
   PGPError __fastcall PGPOpenDefaultKeyRings(PGPContextRef context, PGPKeyRingOpenFlags openFlags,
      PGPKeySetRef *keySet);
   PGPError __fastcall PGPNewKeyDisabledFilter(PGPContextRef context, PGPBoolean disabled,
      PGPFilterRef *outFilter);
   PGPError __fastcall PGPNewKeyRevokedFilter(PGPContextRef context, PGPBoolean revoked,
      PGPFilterRef *outFilter);
   PGPError __fastcall PGPFilterKeySet(PGPKeySetRef origSet, PGPFilterRef filter,
      PGPKeySetRef *resultSet);
   PGPError __fastcall PGPOrderKeySet(PGPKeySetRef src, PGPKeyOrdering order,
      PGPKeyListRef *outRef );
   PGPError __fastcall PGPNewKeyIter(PGPKeyListRef keys, PGPKeyIterRef *outRef);
   PGPError __fastcall PGPGetHashAlgUsed(PGPKeyRef key, PGPHashAlgorithm *hashAlg );
   PGPError __fastcall PGPGetKeyIDFromKey(PGPKeyRef key, PGPKeyID *id);
   PGPError __fastcall PGPGetKeyIDFromString(const char *string, PGPKeyID *id);
   PGPError __fastcall PGPGetKeyIDString(PGPKeyID const *ref, PGPKeyIDStringType type,
      char outString[kPGPMaxKeyIDStringSize]);
   PGPError __fastcall PGPKeyIterNext(PGPKeyIterRef iter, PGPKeyRef *outRef);
   PGPError __fastcall PGPFreeKeyIter(PGPKeyIterRef iter);
   PGPError __fastcall PGPFreeKeyList(PGPKeyListRef keys);
   PGPError __fastcall PGPFreeFilter(PGPFilterRef filter);
   PGPError __fastcall PGPNewFileSpecFromFullPath( PGPContextRef context, char const *path, PGPFileSpecRef *ref);
   PGPError __fastcall PGPOpenKeyRing(PGPContextRef context, PGPKeyRingOpenFlags openFlags,
      PGPFileSpecRef fileRef, PGPKeySetRef *keySet);
   PGPError __fastcall PGPDecodeSig(PGPContextRef context, PGPOptionListRef firstOption,
      PGPOptionListRef secondOption, PGPOptionListRef thirdOption,
      PGPOptionListRef forthOption, PGPOptionListRef fifthOption);
   PGPError	__fastcall PGPOpenKeyRingPair(PGPContextRef context, PGPKeyRingOpenFlags openFlags,
      PGPFileSpecRef pubFileRef, PGPFileSpecRef secFileRef, PGPKeySetRef *keySet);
   PGPError __fastcall PGPGetKeyByKeyID(PGPKeySetRef keySet, PGPKeyID const *keyID,
      PGPPublicKeyAlgorithm pubKeyAlgorithm, PGPKeyRef *key );
   PGPError __fastcall PGPDecode(PGPContextRef context, PGPOptionListRef opt1,
      PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
      PGPOptionListRef opt5, PGPOptionListRef opt6, PGPOptionListRef opt7,
      PGPOptionListRef opt8, PGPOptionListRef opt9);
   PGPError __fastcall PGPEncode(PGPContextRef context, PGPOptionListRef opt1,
      PGPOptionListRef opt2, PGPOptionListRef opt3, PGPOptionListRef opt4,
      PGPOptionListRef opt5, PGPOptionListRef opt6, PGPOptionListRef opt7,
      PGPOptionListRef opt8, PGPOptionListRef opt9, PGPOptionListRef opt10,
      PGPOptionListRef opt11, PGPOptionListRef opt12, PGPOptionListRef opt13,
      PGPOptionListRef opt14, PGPOptionListRef opt15,
      PGPOptionListRef opt16);
   PGPError __fastcall PGPGetPrimaryUserID(PGPKeyRef key, PGPUserIDRef *userID);
   PGPError __fastcall PGPGetUserIDStringBuffer(PGPUserIDRef userID,
      PGPUserIDPropName propName, PGPSize availLength, char *propString, PGPSize *usedLength );
   static PGPError SigEventHandler(PGPContextRef context, PGPEvent *event,
      PGPUserValue userValue);
   static PGPError DecodeEventHandler(PGPContextRef context, PGPEvent *event,
      PGPUserValue userValue);
   AnsiString __fastcall TimeT2SigTime(time_t thetime);
   PGPOptionListRef __fastcall PGPOExportFormat(PGPContextRef context, PGPExportFormat exportFormat);
   PGPOptionListRef __fastcall PGPOExportPrivateKeys(PGPContextRef context, PGPBoolean exportKeys);
   PGPOptionListRef __fastcall PGPOVersionString(PGPContextRef context, const char *version);
   PGPOptionListRef __fastcall PGPOLastOption(PGPContextRef context);
   PGPOptionListRef __fastcall PGPOAllocatedOutputBuffer(PGPContextRef context, void **buffer,
      PGPSize maximumBufferSize, PGPSize *actualBufferSize);
   PGPOptionListRef __fastcall PGPOInputBuffer( PGPContextRef context, void const *buffer,
      PGPSize bufferSize);
   PGPOptionListRef __fastcall PGPOKeySetRef(PGPContextRef context, PGPKeySetRef keysetRef);
   PGPOptionListRef	__fastcall PGPOEventHandler(PGPContextRef context,
       PGPEventHandlerProcPtr eventHandler, PGPUserValue eventHandlerData);
   PGPOptionListRef __fastcall PGPORecursivelyDecode(PGPContextRef context, PGPBoolean recurse);
   PGPOptionListRef __fastcall PGPOPassphrase(PGPContextRef context, const char *passphrase);
   PGPOptionListRef __fastcall PGPOClearSign(PGPContextRef context, PGPBoolean);
   PGPOptionListRef __fastcall PGPOSignWithKey(PGPContextRef context, PGPKeyRef SignKey,
      PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3);
   PGPBoolean __fastcall PGPPassphraseIsValid(PGPKeyRef key, PGPOptionListRef opt1,
      PGPOptionListRef opt2);
   PGPError __fastcall PGPNewKeySet(PGPContextRef Context,PGPKeySetRef *keySet);
   PGPError __fastcall PGPNewSingletonKeySet(PGPKeyRef key, PGPKeySetRef *keySet);
   PGPError __fastcall PGPAddKeys(PGPKeySetRef keysToAdd, PGPKeySetRef keySet);
   PGPOptionListRef __fastcall PGPOArmorOutput(PGPContextRef Context,
      PGPBoolean armorOutput);
   PGPOptionListRef __fastcall PGPOEncryptToKeySet(PGPContextRef Context,
      PGPKeySetRef keySet );
   PGPOptionListRef __fastcall PGPOCompression(PGPContextRef pgpContext,
      PGPBoolean isCompressed );
   PGPOptionListRef __fastcall PGPOConventionalEncrypt(PGPContextRef Context,
      PGPOptionListRef opt1, PGPOptionListRef opt2, PGPOptionListRef opt3);
   PGPOptionListRef __fastcall PGPOOutputLineEndType(PGPContextRef pgpContext,
      PGPLineEndType lineEndType );
   PGPOptionListRef __fastcall PGPODataIsASCII(PGPContextRef pgpContext,
      PGPBoolean dataIsASCII );
   PGPOptionListRef __fastcall PGPOForYourEyesOnly(PGPContextRef pgpContext,
      PGPBoolean forYourEyesOnly);
   PGPOptionListRef __fastcall PGPOHashAlgorithm(PGPContextRef pgpContext,
      PGPHashAlgorithm algID );
   void __fastcall ShowPGPError(PGPError err, AnsiString errmsg);
   PGPError __fastcall PGPCommitKeyRingChanges( PGPKeySetRef keySet );
   time_t __fastcall PGPGetStdTimeFromPGPTime(PGPTime theTime);
   PGPTime __fastcall PGPGetTime();
};

typedef PGPError (*PGPSDKINIT)(void);
typedef PGPError (*PGPSDKCLEANUP)(void);
typedef PGPError (*PGPNEWCONTEXT)(PGPUInt32, PGPContextRef*);
typedef PGPError (*PGPSDKLOADDEFAULTPREFS)(PGPContextRef);
typedef PGPError (*PGPSDKPREFGETFILESPEC)(PGPContextRef, PGPsdkPrefSelector, PGPFileSpecRef*);
typedef PGPError (*PGPGETFULLPATHFROMFILESPEC)(PGPFileSpecRef, char**);
typedef PGPError (*PGPGETERRORSTRING)(PGPError, PGPSize, char*);
typedef PGPError (*PGPGETSDKVERSION)(PGPUInt32*);
typedef PGPError (*PGPFREEFILESPEC)(PGPFileSpecRef fileRef);
typedef PGPError (*PGPEXPORTKEYSET)(PGPKeySetRef, PGPOptionListRef, PGPOptionListRef,
      PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPFREEKEYSET)(PGPKeySetRef);
typedef PGPError (*PGPFREEDATA)(void *allocation);
typedef PGPError (*PGPOPENDEFAULTKEYRINGS)(PGPContextRef, PGPKeyRingOpenFlags, PGPKeySetRef*);
typedef PGPError (*PGPNEWKEYDISABLEDFILTER)(PGPContextRef, PGPBoolean, PGPFilterRef*);
typedef PGPError (*PGPNEWKEYREVOKEDFILTER)(PGPContextRef, PGPBoolean, PGPFilterRef*);
typedef PGPError (*PGPFILTERKEYSET)(PGPKeySetRef, PGPFilterRef, PGPKeySetRef*);
typedef PGPError (*PGPORDERKEYSET)(PGPKeySetRef, PGPKeyOrdering, PGPKeyListRef*);
typedef PGPError (*PGPNEWKEYITER)(PGPKeyListRef, PGPKeyIterRef*);
typedef PGPError (*PGPGETHASHALGUSED)(PGPKeyRef key, PGPHashAlgorithm *hashAlg );
typedef PGPError (*PGPGETKEYIDFROMKEY)(PGPKeyRef, PGPKeyID*);
typedef PGPError (*PGPGETKEYIDFROMSTRING)(const char*, PGPKeyID*);
typedef PGPError (*PGPGETKEYIDSTRING)(PGPKeyID const*, PGPKeyIDStringType,
   char[kPGPMaxKeyIDStringSize]);
typedef PGPError (*PGPKEYITERNEXT)(PGPKeyIterRef, PGPKeyRef*);
typedef PGPError (*PGPFREEKEYITER)(PGPKeyIterRef);
typedef PGPError (*PGPFREEKEYLIST)(PGPKeyListRef);
typedef PGPError (*PGPFREEFILTER)(PGPFilterRef);
typedef PGPError (*PGPNEWFILESPECFROMFULLPATH)(PGPContextRef, const char*, PGPFileSpecRef*);
typedef PGPError (*PGPOPENKEYRING)(PGPContextRef, PGPKeyRingOpenFlags, PGPFileSpecRef,
   PGPKeySetRef*);
typedef PGPError (*PGPGETKEYBYKEYID)(PGPKeySetRef, PGPKeyID const*, PGPPublicKeyAlgorithm,
   PGPKeyRef*);
typedef PGPError (*PGPDECODESIG)(PGPContextRef context, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPDECODE)(PGPContextRef context, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPENCODE)(PGPContextRef context, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef);
typedef PGPError (*PGPGETPRIMARYUSERID)(PGPKeyRef, PGPUserIDRef*);
typedef PGPError (*PGPGETUSERIDSTRINGBUFFER)(PGPUserIDRef, PGPUserIDPropName, PGPSize,
   char*, PGPSize*);
typedef PGPError (*PGPOPENKEYRINGPAIR)(PGPContextRef, PGPKeyRingOpenFlags, PGPFileSpecRef,
   PGPFileSpecRef, PGPKeySetRef*);
typedef PGPError (*PGPNEWKEYSET)(PGPContextRef, PGPKeySetRef*);
typedef PGPError (*PGPNEWSINGLETONKEYSET)(PGPKeyRef, PGPKeySetRef*);
typedef PGPError (*PGPADDKEYS)(PGPKeySetRef, PGPKeySetRef);
typedef PGPError (*PGPCOMMITKEYRINGCHANGES)(PGPKeySetRef);
typedef PGPOptionListRef (*PGPOEXPORTFORMAT)(PGPContextRef, PGPExportFormat);
typedef PGPOptionListRef (*PGPOEXPORTPRIVATEKEYS)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOVERSIONSTRING)(PGPContextRef, const char*);
typedef PGPOptionListRef (*PGPOLASTOPTION)(PGPContextRef);
typedef PGPOptionListRef (*PGPOALLOCATEDOUTPUTBUFFER)(PGPContextRef, void**, PGPSize,
   PGPSize*);
typedef PGPOptionListRef (*PGPOINPUTBUFFER)(PGPContextRef, void const*, PGPSize);
typedef PGPOptionListRef (*PGPOKEYSETREF)(PGPContextRef context, PGPKeySetRef keysetRef);
typedef PGPOptionListRef (*PGPOEVENTHANDLER)(PGPContextRef, PGPEventHandlerProcPtr,
   PGPUserValue);
typedef PGPOptionListRef (*PGPORECURSIVELYDECODE)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOPASSPHRASE)(PGPContextRef, const char*);
typedef PGPOptionListRef (*PGPOCLEARSIGN)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOSIGNWITHKEY)(PGPContextRef, PGPKeyRef, PGPOptionListRef,
   PGPOptionListRef, PGPOptionListRef);
typedef PGPOptionListRef (*PGPOCOMPRESSION)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOARMOROUTPUT)(PGPContextRef, PGPBoolean);
typedef PGPOptionListRef (*PGPOENCRYPTTOKEYSET)(PGPContextRef,PGPKeySetRef);
typedef PGPOptionListRef (*PGPOCONVENTIONALENCRYPT)(PGPContextRef,
   PGPOptionListRef, PGPOptionListRef, PGPOptionListRef);
typedef PGPOptionListRef (*PGPOOUTPUTLINEENDTYPE)(PGPContextRef,PGPLineEndType);
typedef PGPOptionListRef (*PGPODDATAISSCII)(PGPContextRef,PGPBoolean);
typedef PGPOptionListRef (*PGPOFORYOUREYESONLY)(PGPContextRef pgpContext, PGPBoolean forYourEyesOnly);
typedef PGPOptionListRef (*PGPOHASHALGORITHM)(PGPContextRef pgpContext, PGPHashAlgorithm algID );

void __fastcall ShowPGPError(PGPError err, AnsiString errmsg);
typedef PGPBoolean (*PGPPASSPHRASEISVALID)(PGPKeyRef, PGPOptionListRef, PGPOptionListRef);
typedef time_t (*PGPGETSTDTIMEFROMPGPTIME)(PGPTime);
typedef PGPTime (*PGPGETTIME)();

//---------------------------------------------------------------------------
extern TPGPsdk *PGPsdk;
extern HINSTANCE hPgpSdk;
//---------------------------------------------------------------------------
#endif
